<?php
/*
  Plugin Name: GA-PVcounter 
  Plugin URI: http://celtislab.net/wp_plugin_GA-PVcounter
  Description: Google Analytics PageView counter and Popular posts and Recent posts widget 
  Author: enomoto@celtislab
  Version: 0.4.0  
  Author URI: http://celtislab.net/
  Text Domain: GA-PVcounter
  Domain Path: /languages
------------------------------------------------------------ 

使用ライブラリ
Google APIs Client Library for PHP
https://code.google.com/p/google-api-php-client/

使い方
WordPress の plugins フォルダに GA-PVcounter フォルダを作成して、このファイル GA-PVcounter.php をコピーします
管理画面のプラグイン一覧から有効化して下さい
プラグインをインストールしたら管理モードの GA-PVcount 設定画面を開きます
この画面には、次に作成する ID キーに設定するリダイレクト先のURIが表示されています

Google APIs Console　https://code.google.com/apis/console?api=analytics サイトをブラウザの新しいウィンドウで開きます
Analytics アクセスに必要な IDキーを作成します
Redirect URIs には、GA-PVcount 設定画面に表示されているURIをセットします

生成した  client id, client secret, DeveloperKey を GA-PVcounter プラグインのログイン処理画面で設定します
Login ボタンをクリックして OAuth 2.0 認証を行います

--------------------------------------------------------------
ver 0.1.0 2013/8/6 test release

ver 0.2.0 2013/8/9
   WordPress3.6 jquery 1.10 support
 　サイドバーに複数配置しても正常に表示するように対策
 　JavaScript を別ファイルへ分ける
   人気記事の最大表示数　０に対応 (PV Chart のみ表示）
 
 ver 0.3.0 2013/8/20
   設定画面からプロファイルIDを指定できるように修正
   オプションにテキストウィジェットでのショートコード許可を追加
 
 ver 0.4.0 2013/10/1
   最新記事ウィジェット追加
   サムネイル画像表示設定追加及びレイアウト変更
 　スモールPVチャート表示設定
   人気記事一覧ページへのリンク設定エリア追加
   CSS調整
 
 */

const GA_PVC_POPULARPOSTS = 100;         //最大記事取得数
const GA_PVC_RECENTPOSTS  = 10;          //最新記事取得数

require_once 'GA-PVwidget.php';

$ga_pvcounter = new GA_PVcounter();

class GA_PVcounter {

    const   DOMAIN = 'GA-PVcounter';    //翻訳用ドメイン定義
    
    public $goption = null;
    public $gaccess = null;
    public $grecent = null;

    public function __construct() {

        load_plugin_textdomain(self::DOMAIN, false, basename( dirname( __FILE__ ) ).'/languages' );
        
        //アクセスキーデータ
        $default = array( 'token' => null, 'pid' => null, 'pname' => null, 'website' => null, 'day' => null, 'time' => null, 'result' => null  );
		$access = wp_parse_args( (array)get_option('ga_pvc_access'),  $default);
        //オプションデータ
        $default = array( 'clientid' => '', 'clientsecret' => '', 'apikey' => '', 'profile' => '', 'interval' => 10, 'twshortcode' => false );
        $default['redirect'] = admin_url('options-general.php?page=GA-PVcounter/GA-PVcounter.php');
        $instance = $this->my_update_option( get_option('ga_pvc_option'));    // update post
		$option   = wp_parse_args( (array) $instance,  $default);
        //最新記事データ
		$recent  = get_option('ga_pvc_recent');
        
        //問い合わせ条件（指定間隔(分）以上経っている場合のみ有効）
        $queryfg = false;
        if (isset($access['token']) && isset($access['pid'])) {
            $now = strtotime("now");
            if(!isset($access['time']))
                $queryfg = true;
            else if( $access['time'] < $now)
                $queryfg = true;
        }
        $requri = $_SERVER['REQUEST_URI'];
        if( (!is_admin() && $queryfg) || stripos($requri, 'GA-PVcounter.php' ) !== false ){
            //Googleのライブラリが別プラグイン(google-analycator) と競合しているようだ
            //ウィジェット側ではライブラリを使用しない処理に限定するほうが無難か
            //とりあえず、必要な時だけ読みこむようにしたら動作可能
            if(!class_exists('Google_Client', FALSE)) { 
                require_once __DIR__.'/google-api-php-client/src/Google_Client.php';
            }
            if(!class_exists('Google_AnalyticsService', FALSE)) { 
                require_once __DIR__.'/google-api-php-client/src/contrib/Google_AnalyticsService.php';
            }
            $client = new Google_Client();
            $client->setApplicationName('Google Analytics PV counter widget');
            $client->setClientId($option['clientid']);
            $client->setClientSecret($option['clientsecret']);
            $client->setRedirectUri($option['redirect']);
            $client->setDeveloperKey($option['apikey']);
            $client->setScopes(array('https://www.googleapis.com/auth/analytics.readonly'));
            $client->setUseObjects(true);
        }

        if (isset($access['token'])) {
            //ログアウトクリック時
            if (isset($_REQUEST['submit_logout'])) {
                $uri = $option['redirect'];
                delete_option('ga_pvc_access');
                delete_option('ga_pvc_option');
                delete_option('ga_pvc_recent');
                $client->revokeToken();
                header('Location: ' . $uri);
                exit;
            }
            //オプション更新時に結果クリア
            if (isset($_REQUEST['submit_save'])) {
                $access['day']  = null;
                $access['time'] = null;
                $access['result'] = null;
                update_option('ga_pvc_access', $access );
                delete_option('ga_pvc_recent');
            }
        }
        else {
            //ログインクリック時 oauth 認証へリダイレクト
            if (isset($_REQUEST['submit_login'])){
                if (!empty($option['clientid']) && !empty($option['clientsecret']) && !empty($option['apikey'])) {
                    $authUrl = $client->createAuthUrl();
                    header("Location: " . $authUrl);
                    exit;
                }
            }
            //oauth 認証により　アクセストークン取得
            if (isset($_GET['code'])) {
                $client->authenticate();
                $access['token'] = $client->getAccessToken();
                update_option('ga_pvc_access', $access );
                header('Location: ' . $option['redirect']);
                exit;
            }
        }
            
        add_action('widgets_init', array(&$this, 'my_register_wedgets'));
        add_shortcode('GA_PVC', array(&$this, 'GA_PVC_shortcode'));

        if($option['twshortcode'] === true)
            add_filter('widget_text', 'do_shortcode');  //テキストウィジェットでショートコード使用許可

        if(is_admin()) {
            //管理画面
            add_action('admin_menu', array(&$this, 'my_option_menu'));
            add_action('admin_init', array(&$this, 'my_option_register'));

            //プラグイン削除時のフック(コールバック関数を static にする必要あり）
            if ( function_exists('register_uninstall_hook') )
                register_uninstall_hook(__FILE__, 'GA_PVcounter::my_uninstall_hook');

            if (isset($access['token'])) {
                //プロファイル取得済みか
                //if (!isset($access['pid'])) {
                if ($access['pid'] !== $option['profile']) {
                    //アクセストークン　セット
                    $client->setAccessToken($access['token']);
                    //設定画面からの変更は念のためリフレッシュしてから実行
                    $token= json_decode($access['token']);
                    $client->refreshToken($token->refresh_token);
                    $access['token'] = $client->getAccessToken();
                    update_option('ga_pvc_access', $access );
                    
                    if ($client->getAccessToken()) {
                        //登録データのプロファイルを取得
                        $analytics = new Google_AnalyticsService($client);
                        $profile = $this->getFirstProfile($analytics, $option['profile']);
                        if (isset($profile)) {
                            $access['pid']     = $profile->getID();
                            $access['pname']   = $profile->getName();
                            $access['website'] = $profile->getWebsiteUrl();
                            update_option('ga_pvc_access', $access );

                            $option['profile'] = $access['pid'];
                        }
                        else {
                            //プロファイルが取れない場合はトークン以外クリア
                            $acs['token'] = $access['token'];
                            update_option('ga_pvc_access', $acs );
                        }
                    }
                }
            }
        }
        else {            
            //アナリティクスデータを取得したらデータを保持しておく
            if($queryfg){
                //トークンの期限チェック
                $client->setAccessToken($access['token']);
                $token= json_decode($access['token']);
                if($now - 60  > $token->created + $token->expires_in) {
                    //トークンリフレッシュ
                    $client->refreshToken($token->refresh_token);
                    $access['token'] = $client->getAccessToken();
                    update_option('ga_pvc_access', $access );
                }

                // アナリティクスデータ問い合わせ
                if ($client->getAccessToken()) {
                    try {
                        $analytics = new Google_AnalyticsService($client);
                        $result = $this->getResults($analytics, $access, $recent);
                    }
                    catch (apiServiceException $e) {
                        //print 'There was an API error : ' . $e->getCode() . ' : ' . $e->getMessage();
                    } 
                    catch (Exception $e) {
                        //print 'There wan a general error : ' . $e->getMessage();
                    }
                    if(!empty($result))
                        $access['result'] = $result;

                    //問い合わせインターバル時間更新
                    $interval = "now +" .$option['interval']." min";
                    $access['time'] = strtotime( $interval);
                    update_option('ga_pvc_access', $access );
                }
            }
        }
        $this->goption = $option;
        $this->gaccess = $access;
        $this->grecent = $recent;
    }

    //設定オプション表示（メニュー）
    public function my_option_menu()
    {
        $page = add_options_page( 'GA-PVcounter Settings', __('GA-PVcounter'), 'manage_options', __FILE__, array(&$this,'ga_pvc_options'));
    }

    //設定オプション更新  
    //  引数１：グループ名(settings_fields関数の引数で使用する)
    //  引数２：オプション名(input要素などのname属性で使用する）
    public function my_option_register()
    {
        register_setting('ga_pvc_optiongroup', 'ga_pvc_option');
    }

    //プラグイン削除時のオプションクリア
    public static function my_uninstall_hook()
    {
        delete_option('ga_pvc_access');
        delete_option('ga_pvc_option');
    }

    //オプションデータの確認と更新
    public function my_update_option($option)
    {
        $opt = $option;
        // Login クリック処理の　＄_POST の client id を処理する
        if(isset($_POST['submit_login'])){
            if(isset($_POST['ga_pvc_option']['clientid'])){
                $opt['clientid'] = stripslashes($_POST['ga_pvc_option']['clientid']);
            }
            if(isset($_POST['ga_pvc_option']['clientsecret'])){
                $opt['clientsecret'] = stripslashes($_POST['ga_pvc_option']['clientsecret']);
            }
            if(isset($_POST['ga_pvc_option']['apikey'])){
                $opt['apikey'] = stripslashes($_POST['ga_pvc_option']['apikey']);
            }
            if(!empty($opt))
                update_option('ga_pvc_option', $opt );
        }
        //データ更新時の有効性チェック
        if(isset($_POST['submit_save'])){
            if(isset($_POST['ga_pvc_option']['profile'])){
                $opt['profile'] = stripslashes($_POST['ga_pvc_option']['profile']);
            }
            $opt['interval']    = (intval($_POST['ga_pvc_option']['interval']) > 0 ) ? intval( $_POST['ga_pvc_option']['interval'] ) : 10;
            $opt['twshortcode'] = (!isset($_POST['ga_pvc_option']['twshortcode'])) ? false : true; 
            if(!empty($opt))
                update_option('ga_pvc_option', $opt );
        }
        return $opt;
    }
    
    //管理モードのオプション設定画面
    public function ga_pvc_options() {
       ?>
          <div class="wrap">
            <div id="icon-options-general" class="icon32"></div>
            <h2><?php _e( 'Google Analytics PV counter Setting', self::DOMAIN ); ?></h2>
            <p><br /></p>
            <?php

            $access = $this->gaccess;
            $option = $this->goption;

            //ログイン処理
            if (!isset($access['token']) || !isset($access['pid'])) {
            ?>
                <form method="post" action="">
                <?php  settings_fields('ga_pvc_optiongroup');  ?>

                <div class="ga-pvc-guide">
                <p><?php _e( 'You need to authenticate with the Google OAuth 2.0 to access a data analytics', self::DOMAIN );//アナリティクスデータへアクセスするために Google OAuth 2.0 で認証を行う必要があります ?></p>
                <p><strong><?php _e( 'Preparation', self::DOMAIN );//認証準備 ?></strong></p>
                </div>
                <table class="form-table">
                    <tr valign="top">
                        <td><?php _e('Step1', self::DOMAIN); ?></td>
                        <td><?php _e('<a href="https://code.google.com/apis/console?api=analytics" target="_blank">Google APIs Console </a> open a new window of the browser ', self::DOMAIN); //サイトをブラウザの新しいウィンドウで開きます ?></td>
                    </tr>
                    <tr valign="top">
                        <td><?php _e('Step2', self::DOMAIN); ?></td>
                        <td><?php _e('Create the API Project.（Ref. <a href="http://code.google.com/apis/console-help/#UsingOAuth2" target="_blank">Google APIs Console Help</a>)', self::DOMAIN); //API Project を作成します。 ?>
                            <p><?php _e('If you have an existing project, select the Other projects -> Create from API Project drop-down menu', self::DOMAIN);//既存のプロジェクトがある場合は、API Project ドロップダウン メニューから Other projects > Create を選択します ?></P>
                        </td>
                    </tr>
                    <tr valign="top">
                        <td><?php _e('Step3', self::DOMAIN); ?></td>
                        <td><?php _e('Set to ON the Status of Analytics API Service', self::DOMAIN);//Analytics API Service の Status を ON に設定します ?></td>
                    </tr>
                    <tr valign="top">
                        <td><?php _e('Step4', self::DOMAIN); ?></td>
                        <td><?php _e('Register the Home Page URL and Product name by clicking on the Create an OAuth 2.0 client ID button', self::DOMAIN);//Create an OAuth 2.0 client ID ボタンをクリックして Product name と Home Page URL を登録します ?></td>
                    </tr>
                    <tr valign="top">
                        <td><?php _e('Step5', self::DOMAIN); ?></td>
                        <td><?php _e('Creating a Client ID. Select the Web application, you can create a Client ID to register your site', self::DOMAIN); //Client ID の作成。Web application を選択して、あなたのサイトを登録して Client ID を作成します?></td>
                    </tr>
                    <tr valign="top">
                        <td><?php _e('Step6', self::DOMAIN); ?></td>
                        <td><?php _e('Edit from the Edit settings <strong>Redirect URIs</strong> of the Client ID for web applications that created', self::DOMAIN);//作成した Client ID for web applications の <strong>Redirect URIs</strong> を Edit settings から編集します。 ?>
                            <p><?php  _e(" Copy the following URI, and paste it to <strong> Redirect URIs</ strong>", self::DOMAIN); //下記 URI をコピーして <strong>Redirect URIs</strong> へ貼り付けて下さい ?></p>
                            <p><mark><?php  echo $option['redirect'];  ?></mark></p>
                        </td>
                    </tr>
                </table>
                <div class="ga-pvc-guide">
                <p><strong><?php _e( 'Authentication', self::DOMAIN ); //認証 ?></strong></p>
                </div>
                <table class="form-table">
                    <tr valign="top">
                        <td><?php _e('The authentication work using the code <strong>Client ID, Client secret, API Key</strong> of Client ID for web applications and Simple API Access', self::DOMAIN); // Client ID for web applications の <strong>Client ID, Client secret, API Key</strong> のコードを使い認証作業を行います ?></td>
                    </tr>
                </table>
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row"><label for="ga-pvc-clientid"><?php _e( 'Client ID ' ); ?></label></th>
                        <td><input name="ga_pvc_option[clientid]" type="text" size="80" id="ga-pvc-clientid" value="<?php echo $option['clientid']; ?>" /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><label for="ga-pvc-clientsecret"><?php _e( 'Client Secret ' ); ?></label></th>
                        <td><input name="ga_pvc_option[clientsecret]" type="text" size="80" id="ga-pvc-clientsecret" value="<?php echo $option['clientsecret']; ?>" /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><label for="ga-pvc-apikey"><?php _e( 'API Key ' ); ?></label></th>
                        <td><input name="ga_pvc_option[apikey]" type="text" size="80" id="ga-pvc-apikey" value="<?php echo $option['apikey']; ?>" /></td>
                    </tr>

                    <tr valign="top">
                        <td><p class="submit"> <input type="submit" name="submit_login" class="button-primary" value="<?php _e('Login &raquo;', self::DOMAIN); ?>" /> </p> </td>
                    </tr>
                </table>

                <div class="ga-pvc-guide">
                    <p><?php _e( 'After a successful authentication, and authenticate automatically by the token that you saved on subsequent', self::DOMAIN );//認証が成功したら、次回以降は保存したトークンにより自動的に認証を行います ?></p>
                    <p><?php _e( 'Placing to sidebar GA-PVcounter widget, it displays the popular article and page view chart obtained from Analytics', self::DOMAIN );//GA-PVcounter ウィジェットをサイドバーへ配置すれば、アナリティクスから取得したページビューのチャートと人気記事を表示します ?></p>
                </div>
            </form>
          
        <?php
        } 
        //オプション設定 or ログアウト処理
        else {
//            <!-- submit ボタンクリックで　input 要素の name 属性データを post メソッドで option.php へ送信する -->
//            <form method="post" action="options.php"><?php            
        ?>
            <form method="post" action="">
            <?php  settings_fields('ga_pvc_optiongroup');  ?>
                
            <div class="ga-pvc-guide" style="padding-left: 8px;">
            <?php
            if ($access['pid'] === $option['profile']) {
                $profilemsg = __('Analytics Profile : ', self::DOMAIN ) .$access['pid'] .' - ' .$access['pname'];
            }
            else {
                $profilemsg = __('Analytics Profile data was not found', self::DOMAIN );
            }
            ?>    
            <h3><mark><?php echo $profilemsg; ?></mark></h3>
            </div>

              <table class="form-table">
                <tr valign="top">
                    <th scope="row"><label for="ga-pvc-profile"><?php _e( 'Analytics Profile ID', self::DOMAIN ); ?></label></th>
                    <td><input name="ga_pvc_option[profile]" type="text" id="ga-pvc-profile" value="<?php echo esc_attr( $option['profile'] ); ?>" class="medium-text" /></td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="ga-pvc-interval"><?php _e( 'Minimum query interval (minutes)', self::DOMAIN );//最小クエリー間隔（分） ?></label></th>
                    <td><input name="ga_pvc_option[interval]" type="text" id="ga-pvc-interval" value="<?php echo esc_attr( $option['interval'] ); ?>" class="small-text" /></td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="ga-pvc-twshortcode"><?php _e( 'Short code for use in Text Widget', self::DOMAIN );//テキストウィジェットでのショートコード使用 ?></label></th>
                    <td><label><input name="ga_pvc_option[twshortcode]" type="checkbox"  value="1" <?php checked( $option['twshortcode'], 1 ); ?> /><?php _e( ' Enable', self::DOMAIN ); ?></label></td>
                </tr>
                <tr valign="top">
                    <td>
                        <p class="submit"> 
                            <input type="submit" name="submit_logout" class="button-primary" value="<?php _e('Logout', self::DOMAIN); ?>" /> 
                            &nbsp;
                            <input type="submit" name="submit_save" class="button-primary" value="<?php esc_attr_e('Save Changes') ?>" />
                        </p>
                    </td>
                </tr>
              </table>
            </form>

        <?php
        }
        
      ?>
      </div>
      <?php
   }

   //アナリティクスデータのプロファイルを取得（デフォルトは最初のプロファイル）
    public function getFirstprofile(&$analytics, $changepid = '') {
        
        //アナリティクス　アカウント情報（アカウントID)  
        $accounts = $analytics->management_accounts->listManagementAccounts();
        if (count($accounts->getItems()) > 0) {
            $acitems = $accounts->getItems();
            foreach ($acitems as $acitem) {
                $accountId = $acitem->getId();

                //アナリティクス　ウェブプロパティ情報（プロパティID UA-xxxxxxxx-x）
                $webproperties = $analytics->management_webproperties->listManagementWebproperties($accountId);
                if (count($webproperties->getItems()) > 0) {
                    $wpitems = $webproperties->getItems();
                    foreach ($wpitems as $wpitem) {
                        $webpropertyId = $wpitem->getId();

                        //アカウントIDとプロパティIDからプロファイル情報取得（プロファイルID）
                        $profiles = $analytics->management_profiles->listManagementProfiles($accountId, $webpropertyId);
                        if (count($profiles->getItems()) > 0) {
                            $pfitems = $profiles->getItems();
                            if(empty($changepid))   //プロファイルIDが未指定なら最初のプロファイル情報
                                return $pfitems[0];
                            else {                  //指定プロファイルIDと照合
                                foreach ($pfitems as $pfitem) {
                                    if($pfitem->getID() === $changepid )
                                        return $pfitem;
                                }
                            }
                        }
                    }
                }
            }
        } 
        return null;
    }

    //アナリティクスデータ問い合わせ
    //指定期間のページビューカウント
    public function getPageview(&$analytics, $pid, $sday, $eday) 
    {
        $ids = 'ga:' . $pid;            // プロファイルID
        $start_date = $sday->format("Y-m-d");
        $end_date   = $eday->format("Y-m-d");
        $metrics = 'ga:pageviews';      // メトリクス
        if($sday === $eday)
            $dimensions = 'ga:hour';    // ディメンション
        else {
            $diff = $sday->diff($eday);
            if(31 < (int)$diff->format('%a'))
                $dimensions = 'ga:yearMonth';
            else
                $dimensions = 'ga:date';
        }
        $dimensions .= ',ga:deviceCategory';
        $max_results = '100';           // 最大取得数
                                        // オプション（ 一つの配列にまとめる）
        $optParams = array('dimensions' => $dimensions, 'max-results' => $max_results);
        $pview = $analytics->data_ga->get($ids, $start_date, $end_date, $metrics, $optParams);

        $pvsum = 0;
        $pvdata = null;
        if(isset($pview)){
            if (count($pview->getRows()) > 0) {
                $rows = $pview->getRows();
                $headers = $pview->getColumnHeaders();
                $results = array();
                foreach( $rows as $num1 => $row ) {
                    foreach($headers as $num2 => $header ) {
                        $name = $header->name;
                        if( in_array($name, array('ga:hour', 'ga:yearMonth', 'ga:date') ))
                            $name = 'datetime';
                        $results[ $num1 ][ $name ] = $row[ $num2 ];
                    }
                }
                // PV総数計算とPV遷移データ取得
                $dtsav = null;
                foreach( $results as $result ) {
                    $pvsum += $result['ga:pageviews'];
                    if($dtsav !== $result['datetime']){
                        $dtsav = $result['datetime'];
                        $pvdata[ $dtsav ] = array('desktop' => 0, 'mobile' => 0, 'tablet' => 0);
                    }
                }
                foreach( $results as $result ) {
                    $devdt = $pvdata[$result['datetime']];
                    $devdt[$result['ga:deviceCategory']] = $result['ga:pageviews'];
                    $pvdata[$result['datetime']] = $devdt;
                }
            }
        }
        $getdt['pvsum']  = $pvsum;
        $getdt['pvdata'] = $pvdata;
        return $getdt;
    }
    
    //指定期間の人気記事リスト
    public function getPopularposts(&$analytics, $pid, $sday, $eday, $count, $pvsum=0) 
    {
        $ids = 'ga:' . $pid;            // プロファイルID
        $start_date = $sday->format("Y-m-d");
        $end_date   = $eday->format("Y-m-d");
        $metrics = 'ga:pageviews';      // メトリクス
        $dimensions = 'ga:pageTitle,ga:pagePath';
        $sort = '-ga:pageviews';        // ソート項目, - をつけると降順
        $max_results = $count;          
        $optParams = array('dimensions' => $dimensions, 'sort' => $sort, 'max-results' => $max_results);

        $ptitle = $analytics->data_ga->get($ids, $start_date, $end_date, $metrics, $optParams);

        $ttldata = array();
        if(isset($ptitle)){
            if (count($ptitle->getRows()) > 0) {
                $rows = $ptitle->getRows();
                $headers = $ptitle->getColumnHeaders();
                $results = array();
                foreach( $rows as $num1 => $row ) {
                    foreach($headers as $num2 => $header ) {
                        $results[ $num1 ][ $header->name ] = $row[ $num2 ];
                    }
                }
                foreach( $results as $result ) {
                    $rate      = ($pvsum > 0) ? (int)($result[ 'ga:pageviews' ] * 100 / $pvsum):('');
                    $ttldata[] = array('url' => $result[ 'ga:pagePath' ], 'title' => $result[ 'ga:pageTitle' ], 'pv' => $result[ 'ga:pageviews' ], 'rate' => $rate);
                }
            }
        }
        $getdt['popular'] = $ttldata;
        return $getdt;
    }

    //最新記事の指定期間ページビュー（問い合わせ回数を最大個数 GA_PVC_RECENTPOSTS　で制限する)
    public function getRecentposts(&$analytics, $pid, $sday, $eday, &$pagedata) 
    {
        if(empty($pagedata))
            return;
        
        $ids = 'ga:' . $pid;            // プロファイルID
        $start_date = $sday->format("Y-m-d");
        $end_date   = $eday->format("Y-m-d");
        $metrics = 'ga:pageviews';      // メトリクス
        $dimensions = 'ga:date';
        $max_results = '100';          
        
        $idx = 0;
        foreach ($pagedata as $v) {
            //個別記事のフィルターで長いURLを指定するとうまくいかないので正規表現を使い100文字程度でマッチさせる
            $pagepath = $v['url'];
            if(strlen($pagepath) > 100)
                    $pagepath = substr($pagepath, 0, 50) . '.*' . substr($pagepath, -50);
            $filters = 'ga:pagePath=~' . $pagepath;
            //$filters = 'ga:pagePath==/archives/20130314/netbeans-php%E7%89%88-%E4%BD%BF%E3%81%84%E6%96%B9/index.php';
            $optParams = array('filters' => $filters, 'dimensions' => $dimensions, 'max-results' => $max_results);

            $rpv = $analytics->data_ga->get($ids, $start_date, $end_date, $metrics, $optParams);
            $results = array();
            if(isset($rpv)){
                if (count($rpv->getRows()) > 0) {
                    $rows = $rpv->getRows();
                    $headers = $rpv->getColumnHeaders();
                    foreach( $rows as $num1 => $row ) {
                        foreach($headers as $num2 => $header ) {
                            $results[ $num1 ][ $header->name ] = $row[ $num2 ];
                        }
                    }
//                    $sum = 0;
//                    foreach( $results as $result ) {
//                        $sum += $result[ 'ga:pageviews' ];
//                    }
                }
            }
            $pagedata[$idx]['pv'] = $results;
            $idx++;
        }
        update_option('ga_pvc_recent', $pagedata );
    }

    
    //アナリティクスデータ取得
    public function getResults(&$analytics, &$access, &$recent) 
    {    
        $eday = new DateTime();
        $Myear = new DateTime('-365 day');
        $M30day = new DateTime('-30 day');
        $M7day  = new DateTime('-7 day');
        $M1day  = new DateTime('-1 day');
        $sday = $eday;

        //ラストデータ取得
        $getdt = $access['result'];

        // Today PV get
        $pview = $this->getPageview($analytics, $access['pid'], $sday, $eday);
        // 前回取得データからPV総数の変化をチェック
        $changefg = true;
        if(isset($getdt)){
            if($pview['pvsum'] === $getdt['today_pvsum'])
                $changefg = false;
        }
        if($changefg){
            // Today PVデータ更新
            $getdt['today_pvsum']  = $pview['pvsum'];
            $getdt['today_pvdata'] = $pview['pvdata'];

            // 人気記事リスト取得
            $ptitle = $this->getPopularposts($analytics, $access['pid'], $sday, $eday, GA_PVC_POPULARPOSTS, $pview['pvsum']);
            $getdt['today_popular'] = $ptitle['popular'];
            
            if($access['day'] !== $eday->format("Y-m-d")){
                //-7day データ取得
                $pview = $this->getPageview($analytics, $access['pid'], $M7day, $M1day);
                $getdt['7day_pvsum']  = $pview['pvsum'];
                $getdt['7day_pvdata'] = $pview['pvdata'];
                $ptitle = $this->getPopularposts($analytics, $access['pid'], $M7day, $M1day, GA_PVC_POPULARPOSTS, $pview['pvsum']);
                $getdt['7day_popular'] = $ptitle['popular'];
                
                //-30day データ取得
                $pview = $this->getPageview($analytics, $access['pid'], $M30day, $M1day);
                $getdt['30day_pvsum']  = $pview['pvsum'];
                $getdt['30day_pvdata'] = $pview['pvdata'];
                $ptitle = $this->getPopularposts($analytics, $access['pid'], $M30day, $M1day, GA_PVC_POPULARPOSTS, $pview['pvsum']);
                $getdt['30day_popular'] = $ptitle['popular'];

                //-365day データ取得
                $pview = $this->getPageview($analytics, $access['pid'], $Myear, $M1day);
                $getdt['year_pvsum']  = $pview['pvsum'];
                $getdt['year_pvdata'] = $pview['pvdata'];
                $ptitle = $this->getPopularposts($analytics, $access['pid'], $Myear, $M1day, GA_PVC_POPULARPOSTS, $pview['pvsum']);
                $getdt['year_popular'] = $ptitle['popular'];
                
                $access['day'] = $eday->format("Y-m-d");    //7day,30day,year データ取得日保存
            }

            //最新記事PV取得（問い合わせ回数が多すぎないよう考慮　１０件程度）
            $this->getRecentposts($analytics, $access['pid'], $M30day, $eday, $recent);
        }
        else 
            $getdt = null;
            
        return $getdt;
    }
    
    //ショートコード用の人気記事表示
    public function printpopularResults(&$ardata, $type, $max, $show_image) {
        $acs = $this->gaccess;

        if($type == 'year'){
            $ptitle = $ardata['year_popular'];
            $header = __('Year Popular post', self::DOMAIN );
            $pvhead = " PV : " . $ardata['year_pvsum'];
        }
        else if ($type == '30day'){
            $ptitle = $ardata['30day_popular'];
            $header = __('Last 30days Popular post', self::DOMAIN );
            $pvhead = " PV : " . $ardata['30day_pvsum'];
        }
        else if ($type == '7day'){
            $ptitle = $ardata['7day_popular'];
            $header = __('Last 7days Popular post', self::DOMAIN );
            $pvhead = " PV : " . $ardata['7day_pvsum'];
        }
        else {
            $ptitle = $ardata['today_popular'];
            $header = __('Today Popular post', self::DOMAIN );
            $pvhead = " PV : " . $ardata['today_pvsum'];
        }
        if(isset($ptitle)){
            // 人気記事表示
            $num = 1;
            $string = '';
            $site   = (isset($acs['website']))? $acs['website'] : '';                     
            //$site = 'http://localhost/wordpress'; //for debug
            foreach( $ptitle as $idx ) {
                $linkurl = $site.$idx['url'];
                $title   = $idx['title'];
                $pv      = $idx['pv'];
                $rate    = $idx['rate'];
                $ratestr = (!empty($idx['rate']))? "(".$idx['rate']."%)" : '';
                $string .= "<tr>";
                $string .= "<td>$num</td>";
                
                $string .= "<td><div class='ga-pvc-titlewrapper'>";
                if ( $show_image ){
                    $imageurl = plugins_url('defaultimage.png', __FILE__);  //デフォルト（透過PNG）
                    $thumbnail = '<img class="ga-pvc-image" alt="" src="' .$imageurl .'" title="' .$title .'" />';
                    //自サイトしかポストIDは取得できない
                    $posi = stripos($linkurl, "/index.php");
                    if($posi !== false)
                        $linkurl = substr($linkurl, 0, $posi+1 );
                    //$pid = url_to_postid($linkurl);
                    $pid = $this->url_to_postid_spcorrect($linkurl);
                    if($pid > 0){                        
                        if ( has_post_thumbnail($pid)) {
                            $thumbnail = get_the_post_thumbnail($pid, 'thumbnail', array('class' => 'ga-pvc-image', 'alt' => '' , 'title' => $title) );
                        }
                        else {  //記事内から画像検索
                            $imageurl = $this->get_post_maximage($pid);
                            if(!empty($imageurl))
                                $thumbnail = '<img class="ga-pvc-image" alt="" src="' .$imageurl .'" title="' .$title .'" />';
                        }
                    }
                    $string .= "<a href=$linkurl>$thumbnail</a>";
                }
                $string .= "<p><a href='$linkurl'>$title</a></p>";

                if ( $show_image && !empty($thumbnail) )
                    $string .= "<div class='ga-pvc-graphwrap'>";
                $string .= "<div class ='graph'><span class='bar' style=width:$rate%;></span>$pv&nbsp;$ratestr</div>";
                if ( $show_image && !empty($thumbnail) )
                    $string .= "</div>";
                
                $string .= "</div></td>";
                $string .= "</tr>";
                $num++;
                if($max < $num)
                    break;
            }
            ?>
            <table class='ga-pvc-table'>
                <thead><tr><td></td><td><?php echo $header . " (" .$pvhead . ")"; ?></td></thead></tr>
                <tbody><?php echo $string; ?></tbody>
            </table>
            <?php
        }
    }

   //ショートコード 人気記事の表示（タイプと数を指定）
   //[GA_PVC type="today" count="10" image="true"] 
   //  type = today/7day/30day/year
   //  count = 1 - 100
   public  function GA_PVC_shortcode($atts) 
   {
       $access = $this->gaccess;
       extract(shortcode_atts(array(
              'type'  => 'today',
              'count' => 5,
              'image' => "false",
              ), $atts));
       $dtype = "{$type}";
       $max = (int)"{$count}";
       $show_image = ("{$image}" == "true")? true : false;
       
       if (isset($access['token']) && isset($access['pid']) ) {
            $output =  $this->printpopularResults($access['result'], $dtype, $max, $show_image);
       }
       return $output;
   }      
 
   //指定ポストIDの記事内の最大の画像を取得
    public  function get_post_maximage($postid)
    {
        $imageurl = plugins_url('defaultimage.png', __FILE__);  //デフォルト（透過PNG）
        $pdata = get_post( $postid );
//        if( preg_match_all('/<img.+src=[\'"]([^\'"]+)[\'"].*>/i', $pdata->post_content, $matches)){
        if( preg_match_all('/<img.+?src=[\'\"]([^\'"]+)[\'\"].*?>/ui', $pdata->post_content, $matches)){
            //指定サイズが最大の画像
            $maxwidth = 0;
            $maxnum = 0;
            $idx = 0;
            foreach ($matches[0] as $img){
                $match = array();
                if(preg_match('/width=[\'"]([^\'"]+)[\'"]/i', $img, $match)){
                    $width = (int)$match[1];
                    if($maxwidth < $width){
                        $maxwidth = $width;
                        $maxnum  = $idx;
                    }
                }
                $idx++;
            }
            if(!empty($matches[1][$maxnum])){
                $imageurl = $matches[1][$maxnum];
            }
        }
        return $imageurl;
    }

    //URL からポストIDを取得
    //Analytics から取得した URL だと全角スペースが %20 と置き換えられているので %E3%80%80% へ修正 
    public function url_to_postid_spcorrect(&$url)
    {
        $pid = url_to_postid($url);
        if($pid === 0){
            $sp = strpos($url, "%20", 0);
            if($sp !== FALSE){
                //デコードすると通常半角スペースは - に置き換えられているが、%20 となっている（ Analytics のバグか？）
                //半角の空白を全角の空白に置き換えて再度確認
                $durl = urldecode($url);
                $pattern = '/\s/iu';
                $newurl = preg_replace($pattern, '　', $durl);//全角スペース（%E3%80%80% )
                //$eurl = urlencode(mb_convert_encoding($newurl, 'UTF-8', 'auto'));
                $pid = url_to_postid($newurl);
                if($pid !== 0)
                    $url = $newurl;     //URL書き換え
            }
        }
        return $pid;
    }
    
   //ウィジェット有効化
   public function my_register_wedgets()
   {
        register_widget("GA_PVwidget");
        register_widget("GA_PV_recent");
   }
}    

?>
