<?php

/**
 * Description of GA-PVwidget
 *
 * Author: enomoto@celtislab
 * Version: 0.4.0
 * Author URI: http://celtislab.net/
 */


//-------------------------------------------------------------------------
// ウィジェット（保存されているアナリティクスデータを表示するだけ）
//
class GA_PVwidget extends WP_Widget {

    const   DOMAIN = 'GA-PVcounter';    //翻訳用ドメイン定義
    
    function __construct() {
        $name = 'GA-PVcounter';
        $widget_ops = array('classname' => 'GA_PVwidget', 'description' => __('Google Analytics PageView counter and Popular posts', self::DOMAIN));
        parent::__construct( false, $name, $widget_ops );
        
        if(!is_admin()) {
            //<head> 内でCSSファイルを読みこませる
            $urlpath = plugins_url('jquery-ui-1.9.2.custom.css', __FILE__);
            wp_register_style('jquery-ui-custom', $urlpath);
            wp_enqueue_style( 'jquery-ui-custom' );

            $urlpath = plugins_url('GA-PVC-style.css', __FILE__);
            wp_register_style('GA-PVC-style', $urlpath);
            wp_enqueue_style( 'GA-PVC-style' );
            
            wp_enqueue_script( 'jquery' );
            wp_enqueue_script( 'jquery-ui-core' );
            wp_enqueue_script( 'jquery-ui-widget' );
            wp_enqueue_script( 'jquery-ui-tabs' );
            //Google Chart Tools script 読み込み
            wp_enqueue_script( 'jsapi', 'https://www.google.com/jsapi', false, 1, false );
            wp_enqueue_script( 'GA-PVC-handle', plugin_dir_url( __FILE__ ) . 'GA-PVdraw.js', array( 'jquery', 'jsapi' ), false, false );
        }
    }

    function widget( $args, $instance ) {
        global $ga_pvcounter;
        $access = $ga_pvcounter->gaccess;

		extract($args);
        $default  = array( 'title' => '', 'count' => 5, 'show_image'=> false, 'popularpage' => '');
		$instance = wp_parse_args( (array) $instance,  $default);

        $title = apply_filters('widget_title', empty($instance['title']) ? '' : $instance['title']);
        echo $before_widget;
        if ( !empty( $title ) ) {
            echo $before_title . $title . $after_title;
        }
        $tagid = $this->get_field_id('view');
        ?>
    <div  id="<?php echo $tagid; ?>" >
    <?php 
        if (isset($access['token']) && isset($access['pid']) ) {
            $this->printResults($access['result'], $instance['count'], $tagid, $instance['show_image']);
        }
        else {
            _e( '[GA-PVcounter] Can not access the analytics data. Please authenticate from the management screen', self::DOMAIN ); //[GA-PVcounter] アクセス権がありません。管理画面から認証して下さい
        }
    ?>
    </div>
    <?php
        //人気記事一覧ページへのリンク表示
        if(!empty( $instance['popularpage'])){
            echo '<div class="ga-pvc-linkurl" >';
            echo $instance['popularpage'];
            echo '</div>';
        }
    ?>
    <?php
        echo $after_widget;
    }

    function update( $new_instance, $old_instance ) {
        $instance = $old_instance;
        $instance['title'] = strip_tags($new_instance['title']);
		$instance['count'] = (intval($new_instance['count']) >= 0 )  ? intval( $new_instance['count'] ) : 5;
        if($instance['count'] > GA_PVC_POPULARPOSTS)
            $instance['count'] = GA_PVC_POPULARPOSTS;
		$instance['show_image'] = isset( $new_instance['show_image'] ) ? (bool) $new_instance['show_image'] : false;

        if ( current_user_can('unfiltered_html') )  //HTML権限がある場合のみ
			$instance['popularpage'] =  $new_instance['popularpage'];
                
        return $instance;
    }

    function form( $instance ) {
        $default  = array( 'title' => '', 'count' => 5, 'show_image'=> false, 'popularpage' => '');
		$instance = wp_parse_args( (array) $instance,  $default);
        $ttl  = array('id'  => $this->get_field_id('title'),
                      'name'=> $this->get_field_name('title'),
                      'val' => strip_tags($instance['title']) );
        $max  = array('id'  => $this->get_field_id('count'), 
                      'name'=> $this->get_field_name('count'),
                      'val' => $instance['count'] );
		$show_image= isset( $instance['show_image'] ) ? (bool) $instance['show_image'] : false;
        $pglink = array('id'  => $this->get_field_id('popularpage'), 
                        'name'=> $this->get_field_name('popularpage'),
                        'val' => $instance['popularpage'] );
       
    ?>
		<p>
            <label for="<?php echo $ttl['id']; ?>"><?php _e('Title:', self::DOMAIN); ?></label>&nbsp;
            <input class="widefat" id="<?php echo $ttl['id']; ?>" name="<?php echo $ttl['name']; ?>" type="text" value="<?php echo esc_attr($ttl['val']); ?>" />
        </p>
        <p>
            <?php _e('Max number of Popular posts:', self::DOMAIN); ?>
            <input id="<?php echo $max['id']; ?>" name="<?php echo $max['name']; ?>" type="text" value="<?php echo $max['val']; ?>" size="3" />
        </p>
		<p>
            <input class="checkbox" type="checkbox" <?php checked( $show_image ); ?> id="<?php echo $this->get_field_id( 'show_image' ); ?>" name="<?php echo $this->get_field_name( 'show_image' ); ?>" />
            <label for="<?php echo $this->get_field_id( 'show_image' ); ?>"><?php _e( 'Display thumbnail?', self::DOMAIN ); ?></label>
        </p>
        <p>
            <label for="<?php echo $pglink['id']; ?>"><?php _e('Popular post listpage (HTML link tag)', self::DOMAIN); ?></label>&nbsp;
            <textarea class="widefat" rows="5" cols="20" id="<?php echo $pglink['id']; ?>" name="<?php echo $pglink['name']; ?>"><?php echo $pglink['val']; ?></textarea>
        </p>
        
    <?php
    }

    //アナリティクスの問い合わせ結果データを表示
    public function printResults(&$ardata, $max, $tagbase, $show_image) {
        global $ga_pvcounter;
        $acs = $ga_pvcounter->gaccess;

        // PV カウント遷移表示
        $pview[]  = array('day' => 'Today', 'sum' => $ardata['today_pvsum'], 'view' => $ardata['today_pvdata']);
        $pview[]  = array('day' => '7days', 'sum' => $ardata['7day_pvsum'],  'view' => $ardata['7day_pvdata']);
        $pview[]  = array('day' => '30days','sum' => $ardata['30day_pvsum'], 'view' => $ardata['30day_pvdata']);
        $pview[]  = array('day' => 'Year',  'sum' => $ardata['year_pvsum'], 'view' => $ardata['year_pvdata']);

        if(isset($pview)){
            //チャート表示用のデータ配列を作り JavaScript へ渡す為に json 変換
            $pvhed = null;
            $pvcsv = null;
            foreach( $pview as $pv ) {  
                $pvhed[] = $pv['day']." PV : ".$pv['sum'];
                $pvdata  = $pv['view'];

                $arpv = null;
                $arpv[] = array("datetime", "DesktopPV", "MobilePV", "TabletPV");
                foreach( $pvdata as $key => $val ) { 
                    $arpv[] = array( (string)$key, (int)$val['desktop'], (int)$val['mobile'], (int)$val['tablet']);  
                }
                $pvcsv[] = $arpv;
            }
            $pvhedjson = json_encode($pvhed);
            $pviewjson = json_encode($pvcsv);

            //PVチャートを表示させるためのタグ
            $charttag = $tagbase .'-chart';
		?>
            <div id="<?php echo $charttag; ?>"></div>
            <script type='text/javascript'>
            /* <![CDATA[ */
                var ga_pvc_title = <?php echo $pvhedjson; ?>;
                var ga_pvc_data  = <?php echo $pviewjson; ?>;

                google.setOnLoadCallback(ga_pvc_drawChart.bind(this, ga_pvc_title[0], ga_pvc_data[0], '<?php echo $charttag; ?>'));
            /* ]]> */
            </script> 
		<?php
        }

        // 人気記事（jQuery のタブ切り替え表示）
        $ptitle[] = $ardata['today_popular'];
        $ptitle[] = $ardata['7day_popular'];
        $ptitle[] = $ardata['30day_popular'];
        $ptitle[] = $ardata['year_popular'];

        if(isset($ptitle)){
            $ranking = null;
            foreach( $ptitle as $pt ) {  
                $num = 1;
                $string = "<table class='ga-pvc-table'>";
                //$string .= "<thead><tr><td></td><td>Title</td><td>PV</td></thead></tr>";
                $string .= "<tboby>";
                $site   = (isset($acs['website']))? $acs['website'] : '';   
//                $site = 'http://localhost/wordpress'; //for debug
                foreach( $pt as $idx ) {
                    $linkurl = $site.$idx['url'];
                    $title   = $idx['title'];
                    $pv      = $idx['pv'];
                    $rate    = $idx['rate'];
                    $ratestr = (!empty($idx['rate']))? "(".$idx['rate']."%)" : '';
                    $string .= "<tr>";
                    $string .= "<td>$num</td>";
                    
                    $string .= "<td><div class='ga-pvc-titlewrapper'>";
                    if ( $show_image ){
                        $imageurl = plugins_url('defaultimage.png', __FILE__);  //デフォルト（透過PNG）
                        $thumbnail = '<img class="ga-pvc-image" alt="" src="' .$imageurl .'" title="' .$title .'" />';
                        //自サイトしかポストIDは取得できない
                        $posi = stripos($linkurl, "/index.php");
                        if($posi !== false)
                            $linkurl = substr($linkurl, 0, $posi+1 );
                        $pid = $ga_pvcounter->url_to_postid_spcorrect($linkurl);
                        if($pid > 0){                        
                            if ( has_post_thumbnail($pid)) {
                                $thumbnail = get_the_post_thumbnail($pid, 'thumbnail', array('class' => 'ga-pvc-image', 'alt' => '' , 'title' => $title) );
                            }
                            else {  //記事内から画像検索
                                $imageurl = $ga_pvcounter->get_post_maximage($pid);
                                if(!empty($imageurl))
                                    $thumbnail = '<img class="ga-pvc-image" alt="" src="' .$imageurl .'" title="' .$title .'" />';
                            }
                        }
                        $string .= "<a href=$linkurl>$thumbnail</a>";
                    }
                    $string .= "<p><a href='$linkurl'>$title</a></p>";
                    
                    if ( $show_image && !empty($thumbnail) )
                        $string .= "<div class='ga-pvc-graphwrap'>";
                    $string .= "<div class ='graph'><span class='bar' style=width:$rate%;></span>$pv&nbsp;$ratestr</div>";
                    if ( $show_image && !empty($thumbnail) )
                        $string .= "</div>";
                    
                    $string .= "</div></td>";
                    $string .= "</tr>";
                    $num++;
                    if($max < $num)
                        break;
                }
                $string .= "</tboby></table>";
                if($max > 0)
                    $ranking[] = $string;
                else
                    $ranking[] = "";
            }

        ?>
            <div class="ga-pvc-widget-tabs" >
                <ul>
                    <li><a href="#ga-pvc-tab-today">Today</a></li>
                    <li><a href="#ga-pvc-tab-7days">7day</a></li>
                    <li><a href="#ga-pvc-tab-30days">30day</a></li>
                    <li><a href="#ga-pvc-tab-year">Year</a></li>
                </ul>
                <div id="ga-pvc-tab-today">
                    <?php echo $ranking[0]; ?>
                </div>
                <div id="ga-pvc-tab-7days">
                    <?php echo $ranking[1]; ?>
                </div>
                <div id="ga-pvc-tab-30days">
                    <?php echo $ranking[2]; ?>
                </div>
                <div id="ga-pvc-tab-year">
                    <?php echo $ranking[3]; ?>
                </div>
            </div>            
		<?php
        } 
        else {
            ?><p>No results found.</p><?php
        }
    }
}

//-------------------------------------------------------------------------
// ウィジェット（新着記事）
//
class GA_PV_recent extends WP_Widget {

    const   DOMAIN = 'GA-PVcounter';    //翻訳用ドメイン定義

	function __construct() {
        $name = 'GA-PV ' . __('Recent Posts');
		$widget_ops = array('classname' => 'GA_PV_recent', 'description' => __( "Google Analytics PageView counter and Recent posts", self::DOMAIN ) );
        parent::__construct( false, $name, $widget_ops );
        
        if(!is_admin()) {
            //<head> 内でCSSファイルを読みこませる
            $urlpath = plugins_url('GA-PVC-style.css', __FILE__);
            wp_register_style('GA-PVC-style', $urlpath);
            wp_enqueue_style( 'GA-PVC-style' );
            
            wp_enqueue_script( 'jquery' );
            //Google Chart Tools script 読み込み
            wp_enqueue_script( 'jsapi', 'https://www.google.com/jsapi', false, 1, false );
            wp_enqueue_script( 'GA-PVC-handle', plugin_dir_url( __FILE__ ) . 'GA-PVdraw.js', array( 'jquery', 'jsapi' ), false, false );
            
        }
        
	}

	function widget($args, $instance) {
        global $ga_pvcounter;
        $access = $ga_pvcounter->gaccess;
        $recent = $ga_pvcounter->grecent;
        
		extract($args);

		$title = ( ! empty( $instance['title'] ) ) ? $instance['title'] : __( 'Recent Posts' );
		$title = apply_filters( 'widget_title', $title, $instance, $this->id_base );
		$number = ( ! empty( $instance['number'] ) ) ? absint( $instance['number'] ) : 5;
		if ( $number === 0 )
 			$number = 1;
		if ( $number > GA_PVC_RECENTPOSTS )
 			$number = GA_PVC_RECENTPOSTS;
        
		$show_image= isset( $instance['show_image'] ) ? $instance['show_image'] : false;
		$show_date = isset( $instance['show_date'] ) ? $instance['show_date'] : false;
		$show_chart= isset( $instance['show_chart'] ) ? $instance['show_chart'] : false;

		$r = new WP_Query( apply_filters( 'widget_posts_args', array( 'posts_per_page' => GA_PVC_RECENTPOSTS, 'no_found_rows' => true, 'post_status' => 'publish', 'ignore_sticky_posts' => true ) ) );
		$plinks = array();
        if ($r->have_posts()) {
            $site = (isset($access['website']))? $access['website'] : ''; 
            if($site !== get_site_url())
                $site = get_site_url();
            
            while ( $r->have_posts() ) {
                $r->the_post();
                $pid    = get_the_ID();
                $ptitle = get_the_title() ? get_the_title() : $pid;
                $pdate  = get_the_date();
                
                //サイト部分を除外したパーマリンク取得
                $url  = get_permalink();
                if(isset($site)){
                    $start = stripos($url, $site, 0);
                    if($start !== FALSE){
                        $pltop = $start + strlen($site);
                        $url   = substr($url, $pltop);
                    }
                }
                $plinks[] = array('url' => $url, 'id' => $pid, 'title' =>$ptitle, 'date' => $pdate, 'pv' => null);
            }
    		// Reset the global $the_post as this query will have stomped on it
            wp_reset_postdata();
            
            //$recent と同期をとる
            $newrecent = array();
            foreach ($plinks as $plink) {
                $exist = false;
                if(!empty($recent)){
                    foreach ($recent as $v) {
                        if($v['url'] === $plink['url']){
                            $newrecent[] = $v;
                            $exist = true;
                            break;
                        }
                    }
                }
                if(! $exist)
                    $newrecent[] = $plink;
            }
            $recent = $newrecent;
            update_option('ga_pvc_recent', $recent );

            //$recent 内のデータを表示
            echo $before_widget;
            if ( $title )
                echo $before_title . $title . $after_title;

            ?>
            <table class='ga-pvc-table'><tboby>
            <?php
            
                $idx = 0;
                foreach ($recent as $v) {
                    $purl   = $site . $v['url'];
                    $pid    = $v['id'];
                    $ptitle = $v['title'];
                    $pdate  = $v['date'];

                    //サムネイル画像
                    $thumbnail = '';
                    if(isset($pid)){
                        if ( has_post_thumbnail($pid)) {
                            $thumbnail = get_the_post_thumbnail($pid, 'thumbnail', array('class' => 'ga-pvc-image', 'alt' => '' , 'title' => $ptitle) );
                        }
                        else {  //記事内から画像検索          
                            $imageurl = $ga_pvcounter->get_post_maximage($pid); 
                            if(!empty($imageurl))
                                $thumbnail = '<img class="ga-pvc-image" alt="" src="' .$imageurl .'" title="' .$ptitle .'" />';
                        }
                    }
                    $string = "<tr><td><div class='ga-pvc-titlewrapper'>";
                    if ( $show_image && !empty($thumbnail) )
                        $string .= "<a href=$purl>$thumbnail</a>";
                    $string .= "<p><a href=$purl>$ptitle</a>";
                    if ( $show_date )
                        $string .= "<br />$pdate";
                    $string .= "</p>";

                    //PV データ配列作成
                    $pvcsv = null;
                    $pviewjson = null;
                    if(!empty($v['pv'])){
                        $pvcsv[] = array("datetime", "PV");
                        foreach( $v['pv'] as $pv ) {
                            $tpv = $pv[ 'ga:pageviews' ];
                            $pvcsv[] = array($pv['ga:date'], (int)$tpv);
                        }
                        //サムネイル表示有りならマージン設定
                        if ( $show_image && !empty($thumbnail) )
                            $string .= "<div class='ga-pvc-graphwrap'>";
                        $string .= "<div class ='schart'>Today PV:$tpv</div>";
                        if($show_chart) {
                            $pviewjson = json_encode($pvcsv);
                            $charttag = 'ga-pvc-small-' .$pid;
                            $string .= "<div id='" .$charttag . "' class ='schart'></div>";
                        }
                        if ( $show_image && !empty($thumbnail) )
                            $string .= "</div>";
                    }
                    
                    $string .= "</div></td></tr>";
                    echo $string;
                    
                    if(!empty($pviewjson)){
                        ?>
                        <script type='text/javascript'>
                        /* <![CDATA[ */
                            var ga_pvc_scdata  = <?php echo $pviewjson; ?>;
                            google.setOnLoadCallback(ga_pvc_smallChart.bind(this, ga_pvc_scdata, '<?php echo $charttag; ?>'));
                        /* ]]> */
                        </script> 
                        <?php
                    }
                    if(++$idx >= $number)
                        break;
                }
            ?>
            </tboby></table>
            <?php
		
            echo $after_widget;
        }
	}

	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['number'] = (int) $new_instance['number'];
		$instance['show_image'] = isset( $new_instance['show_image'] ) ? (bool) $new_instance['show_image'] : false;
		$instance['show_date']  = isset( $new_instance['show_date'] ) ? (bool) $new_instance['show_date'] : false;
		$instance['show_chart'] = isset( $new_instance['show_chart'] ) ? (bool) $new_instance['show_chart'] : false;

		return $instance;
	}

	function form( $instance ) {
		$title     = isset( $instance['title'] ) ? esc_attr( $instance['title'] ) : '';
		$number    = isset( $instance['number'] ) ? absint( $instance['number'] ) : 5;
		$show_image= isset( $instance['show_image'] ) ? (bool) $instance['show_image'] : false;
		$show_date = isset( $instance['show_date'] ) ? (bool) $instance['show_date'] : false;
		$show_chart= isset( $instance['show_chart'] ) ? (bool) $instance['show_chart'] : false;
?>
		<p><label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:', self::DOMAIN ); ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo $title; ?>" /></p>

		<p><label for="<?php echo $this->get_field_id( 'number' ); ?>"><?php _e( 'Number of posts to show:', self::DOMAIN ); ?></label>
		<input id="<?php echo $this->get_field_id( 'number' ); ?>" name="<?php echo $this->get_field_name( 'number' ); ?>" type="text" value="<?php echo $number; ?>" size="3" /></p>

		<p><input class="checkbox" type="checkbox" <?php checked( $show_image ); ?> id="<?php echo $this->get_field_id( 'show_image' ); ?>" name="<?php echo $this->get_field_name( 'show_image' ); ?>" />
		<label for="<?php echo $this->get_field_id( 'show_image' ); ?>"><?php _e( 'Display thumbnail?', self::DOMAIN ); ?></label></p>

		<p><input class="checkbox" type="checkbox" <?php checked( $show_date ); ?> id="<?php echo $this->get_field_id( 'show_date' ); ?>" name="<?php echo $this->get_field_name( 'show_date' ); ?>" />
		<label for="<?php echo $this->get_field_id( 'show_date' ); ?>"><?php _e( 'Display post date?', self::DOMAIN ); ?></label></p>

		<p><input class="checkbox" type="checkbox" <?php checked( $show_chart ); ?> id="<?php echo $this->get_field_id( 'show_chart' ); ?>" name="<?php echo $this->get_field_name( 'show_chart' ); ?>" />
		<label for="<?php echo $this->get_field_id( 'show_chart' ); ?>"><?php _e( 'Display small PV chart?', self::DOMAIN ); ?></label></p>
<?php
	}
}

?>
